package me.wazup.hideandseek.utils;

import org.bukkit.entity.Player;
import java.util.Iterator;
import java.util.ArrayList;
import org.bukkit.scoreboard.DisplaySlot;
import org.bukkit.Bukkit;
import java.util.List;
import java.util.HashMap;
import org.bukkit.scoreboard.Objective;
import org.bukkit.scoreboard.Scoreboard;

public class FlexibleScoreboard
{
    private final Scoreboard scoreboard;
    private final Objective objective;
    private final HashMap<Integer, LineEntry> lines;
    private final HashMap<String, PlaceholderEntry> placeholders;
    private final HashMap<Integer, List<String>> multiplePlaceholders;
    
    public FlexibleScoreboard(final String s, final boolean b, final String... array) {
        this.lines = new HashMap<Integer, LineEntry>();
        this.placeholders = new HashMap<String, PlaceholderEntry>();
        this.multiplePlaceholders = new HashMap<Integer, List<String>>();
        this.scoreboard = Bukkit.getScoreboardManager().getNewScoreboard();
        (this.objective = this.scoreboard.registerNewObjective("HideAndSeek", "dummy")).setDisplaySlot(DisplaySlot.SIDEBAR);
        this.objective.setDisplayName(this.format(s));
        for (int i = 0; i < array.length; ++i) {
            while (this.scoreboard.getEntries().contains(array[i])) {
                final StringBuilder sb = new StringBuilder();
                final int n = i;
                array[n] = sb.append(array[n]).append(" ").toString();
            }
            final String s2 = array[i];
            final String format = this.format(array[i]);
            final int j = b ? (-(i + 1)) : (array.length - i);
            this.objective.getScore(format).setScore(j);
            this.lines.put(j, new LineEntry(s2, format));
            final int characterCount = this.getCharacterCount(s2, '%');
            if (characterCount > 0 && characterCount % 2 == 0) {
                if (characterCount / 2 > 1) {
                    final ArrayList<String> value = new ArrayList<String>();
                    for (int k = 1; k < characterCount; k += 2) {
                        value.add("%" + s2.split("%")[k] + "%");
                    }
                    this.multiplePlaceholders.put(j, value);
                }
                for (int l = 1; l < characterCount; l += 2) {
                    this.placeholders.put("%" + s2.split("%")[l] + "%", new PlaceholderEntry(j, "N/A"));
                }
            }
        }
    }
    
    public void update(final String s, final int i) {
        this.update(s, String.valueOf(i));
    }
    
    public void update(final String target, final String s) {
        if (!this.placeholders.containsKey(target)) {
            return;
        }
        final PlaceholderEntry placeholderEntry = this.placeholders.get(target);
        placeholderEntry.currentValue = s;
        final int score = placeholderEntry.score;
        final LineEntry lineEntry = this.lines.get(score);
        this.scoreboard.resetScores(lineEntry.currentText);
        String str = lineEntry.originalText;
        if (this.multiplePlaceholders.containsKey(score)) {
            for (final String s2 : this.multiplePlaceholders.get(score)) {
                str = str.replace(s2, this.placeholders.get(s2).currentValue);
            }
        }
        else {
            str = str.replace(target, s);
        }
        while (this.scoreboard.getEntries().contains(str)) {
            str += " ";
        }
        final String format = this.format(str);
        this.objective.getScore(format).setScore(score);
        lineEntry.currentText = format;
    }
    
    public void setNonTrackableScore(final String s, final int score) {
        this.objective.getScore(s).setScore(score);
    }
    
    private int getCharacterCount(final String s, final char c) {
        int n = 0;
        final char[] charArray = s.toCharArray();
        for (int length = charArray.length, i = 0; i < length; ++i) {
            if (charArray[i] == c) {
                ++n;
            }
        }
        return n;
    }
    
    private String format(final String s) {
        return (s.length() > 16) ? (Bukkit.getBukkitVersion().contains("1.7") ? s.substring(0, 16) : ((s.length() > 32) ? s.substring(0, 32) : s)) : s;
    }
    
    public void setName(final String displayName) {
        this.objective.setDisplayName(displayName);
    }
    
    public void apply(final Player player) {
        player.setScoreboard(this.scoreboard);
    }
    
    private class PlaceholderEntry
    {
        int score;
        String currentValue;
        
        private PlaceholderEntry(final int score, final String currentValue) {
            this.score = score;
            this.currentValue = currentValue;
        }
    }
    
    private class LineEntry
    {
        String originalText;
        String currentText;
        
        private LineEntry(final String originalText, final String currentText) {
            this.originalText = originalText;
            this.currentText = currentText;
        }
    }
}
